import json
import os
from typing import List

from crisper_api_client import AuthenticatedClient
from crisper_api_client.api.api import api_v1_assistant_list, api_v1_assistant_retrieve, api_v1_assistant_create
from crisper_api_client.models import Assistant

CRISPER_TOKEN = os.getenv("CRISPER_TOKEN", None)
IS_DEV = os.getenv("ENV", "production") == "development"

class Crisper:
    def __init__(self):
        if not CRISPER_TOKEN:
            raise ValueError("CRISPER_TOKEN is required in the environment variables.")
        base_url = "http://localhost:8000" if IS_DEV else "https://api.crisper.ai"
        self.client = AuthenticatedClient(token=CRISPER_TOKEN, base_url=base_url)
        from crisper_api_client.serivces.assistant_service import AssistantService
        AssistantService.client = self.client

    def assistants(self) -> List[Assistant]:
        response = api_v1_assistant_list.sync_detailed(client=self.client)
        return response.parsed.results

    def get_assistant(self, name: str):
        response = api_v1_assistant_retrieve.sync_detailed(name, client=self.client)
        if response.status_code == 200:
            return Assistant.from_dict(json.loads(response.content))
        return None

    def create_assistant(self, name: str):
        assistant = Assistant(name=name, knowledge_sources=[])
        response = api_v1_assistant_create.sync_detailed(body=assistant, client=self.client)
        to_return = None
        if response.status_code == 201:
            return Assistant.from_dict(json.loads(response.content))
        return to_return




import json
import logging

from crisper_api_client import AuthenticatedClient
from crisper_api_client.api.api import api_v1_assistant_knowledge_source_create, api_v1_assistant_ask_create
from crisper_api_client.models import Assistant, KnowledgeSource, Question, Answer
from crisper_api_client.models.knowledge_source_content_type import KnowledgeSourceContentType


logger = logging.getLogger(__name__)

class AssistantService:
    client: AuthenticatedClient = None

    @classmethod
    def add_knowledge_source(cls, assistant, content_type: str, text: str = '', url: str = ''):
        knowledge_source = KnowledgeSource(
            content_type=KnowledgeSourceContentType(content_type),
            text=text,
            url=url,
        )
        response = api_v1_assistant_knowledge_source_create.sync_detailed(
            name=assistant.name,
            body=knowledge_source,
            client=cls.client
        )
        if response.status_code == 200:
            return Assistant.from_dict(json.loads(response.content))
        return None

    @classmethod
    def ask(cls, assistant, text: str):
        question = Question(text=text)
        response = api_v1_assistant_ask_create.sync_detailed(
            name=assistant.name,
            body=question,
            client=cls.client
        )
        if response.status_code == 200:
            return Answer.from_dict(json.loads(response.content))
        logger.error(f"Error: {response.status_code}")
        return None


Assistant.add_knowledge_source = lambda assistant, **kwargs: AssistantService.add_knowledge_source(assistant, **kwargs)
Assistant.ask = lambda assistant, text: AssistantService.ask(assistant, text)


"""Contains all the data models used in inputs/outputs"""

from .answer import Answer
from .assistant import Assistant
from .knowledge_source import KnowledgeSource
from .knowledge_source_content_type import KnowledgeSourceContentType
from .paginated_assistant_list import PaginatedAssistantList
from .question import Question

__all__ = (
    "Answer",
    "Assistant",
    "KnowledgeSource",
    "KnowledgeSourceContentType",
    "PaginatedAssistantList",
    "Question",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar

from attrs import define as _attrs_define
from attrs import field as _attrs_field

if TYPE_CHECKING:
    from ..models.knowledge_source import KnowledgeSource


T = TypeVar("T", bound="Assistant")


@_attrs_define
class Assistant:
    """
    Attributes:
        knowledge_sources (List['KnowledgeSource']):
        name (str):
    """

    knowledge_sources: List["KnowledgeSource"]
    name: str
    additional_properties: Dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        knowledge_sources = []
        for knowledge_sources_item_data in self.knowledge_sources:
            knowledge_sources_item = knowledge_sources_item_data.to_dict()
            knowledge_sources.append(knowledge_sources_item)

        name = self.name

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "knowledge_sources": knowledge_sources,
                "name": name,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.knowledge_source import KnowledgeSource

        d = src_dict.copy()
        knowledge_sources = []
        _knowledge_sources = d.pop("knowledge_sources")
        for knowledge_sources_item_data in _knowledge_sources:
            knowledge_sources_item = KnowledgeSource.from_dict(knowledge_sources_item_data)

            knowledge_sources.append(knowledge_sources_item)

        name = d.pop("name")

        assistant = cls(
            knowledge_sources=knowledge_sources,
            name=name,
        )

        assistant.additional_properties = d
        return assistant

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
